------------------------------------------------- 
-- Staging Room Screen
-------------------------------------------------
include( "IconSupport" );
include( "SupportFunctions"  );
include( "InstanceManager" );
include( "UniqueBonuses" );


-------------------------------------------------
-- Globals
-------------------------------------------------
local m_SlotInstances = {};
local g_ChatInstances = {};

local g_AdvancedOptionIM = InstanceManager:new( "GameOption", "Text", Controls.AdvancedOptions );
local g_AdvancedOptionsList = {};

local m_HostID;
local m_bIsHost;

local m_PlayerNames = {};
local m_bLaunchReady = false;
local m_bTeamsValid = false;

-------------------------------------------------
-- retrieve player names
-------------------------------------------------
function BuildPlayerNames()
    local playerList = Matchmaking.GetPlayerList();
    
    if( playerList ~= nil ) then
        m_PlayerNames = {};
        
        for i = 1, #playerList do
            m_PlayerNames[ playerList[i].playerID ] = playerList[i].playerName;
        end
    end
end


-------------------------------------------------
-------------------------------------------------
function OnSetCivNames()
    UIManager:PushModal( Controls.SetCivNames );
end
Controls.EditButton:RegisterCallback( Mouse.eLClick, OnSetCivNames );


-------------------------------------------------
-------------------------------------------------
function OnCancel()
	Controls.RemoveButton:SetHide(true);

	PreGame.SetLeaderName( Matchmaking.GetLocalID(), "" );
	PreGame.SetCivilizationDescription( Matchmaking.GetLocalID(), "" );
	PreGame.SetCivilizationShortDescription( Matchmaking.GetLocalID(), "" );
	PreGame.SetCivilizationAdjective( Matchmaking.GetLocalID(), "" );
	
	local civIndex = PreGame.GetCivilization( Matchmaking.GetLocalID() );
    if( civIndex ~= -1 ) then
        civ = GameInfo.Civilizations[ civIndex ];

        -- Use the Civilization_Leaders table to cross reference from this civ to the Leaders table
        local leader = GameInfo.Leaders( "Type = '" .. GameInfo.Civilization_Leaders( "CivilizationType = '" .. civ.Type .. "'" )().LeaderheadType .. "'" )();
		local leaderDescription = leader.Description;
		
		PlayerLeader = Locale.ConvertTextKey( leaderDescription );
		PlayerCiv = Locale.ConvertTextKey( civ.ShortDescription );
		Controls.Title:SetText( Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER_CIV", Locale.ConvertTextKey( leaderDescription ), Locale.ConvertTextKey( civ.ShortDescription ) ) );
	else
		PlayerLeader = Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER" );
        PlayerCiv = Locale.ConvertTextKey( "TXT_KEY_RANDOM_CIV" );
        Controls.Title:SetText( Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER_CIV", PlayerLeader, PlayerCiv ) );

	end
end
Controls.RemoveButton:RegisterCallback( Mouse.eLClick, OnCancel );


-------------------------------------------------
-- Launch Game Button Handler
-------------------------------------------------
function LaunchButtonClick()

	Matchmaking.LaunchMultiplayerGame();

end
Controls.LaunchButton:RegisterCallback( Mouse.eLClick, LaunchButtonClick );


-------------------------------------------------
-- Event Handler: MultiplayerGameLaunched
-------------------------------------------------
function OnGameLaunched()

	UIManager:DequeuePopup( ContextPtr );

end
Events.MultiplayerGameLaunched.Add( OnGameLaunched );


-------------------------------------------------
-------------------------------------------------
function GetPlayerIDBySelectionIndex(selectionIndex)
	if(selectionIndex == 0) then
		return Matchmaking.GetLocalID();
	end

	return m_SlotInstances[selectionIndex].playerID;
end


-------------------------------------------------
-------------------------------------------------
function CivSelected( selectionIndex, civID )
	local playerID = GetPlayerIDBySelectionIndex(selectionIndex);
	if( playerID >= 0 ) then
		PreGame.SetCivilization( playerID, civID );
		Network.BroadcastPlayerInfo();
		UpdateDisplay();
	end
end


-------------------------------------------------
-------------------------------------------------
function InviteSelected( selectionIndex, playerChoiceID )

	local slotInstance = m_SlotInstances[ selectionIndex ];

	if slotInstance then

		if ( playerChoiceID == -1 ) then -- AI

    		slotInstance.InvitePulldown:GetButton():LocalizeAndSetText( "TXT_KEY_AI_NICKNAME" );

		else -- TODO: Send Invite and Lock Slot

			slotInstance.InvitePulldown:GetButton():SetText( Locale.ConvertTextKey("TXT_KEY_WAITING_FOR_INVITE_RESPONSE", "TEMP" ) );
--			slotInstance.InvitePulldown:GetButton():SetText( "Waiting for Player..." );

		end

	end
end


-------------------------------------------------
-------------------------------------------------
function OnKickPlayer( selectionIndex )
	local playerID = GetPlayerIDBySelectionIndex(selectionIndex);
	Matchmaking.KickPlayer( playerID );
	UpdateDisplay();
end


-------------------------------------------------
-------------------------------------------------
function OnReadyCheck( bChecked )
	if(Network.IsEveryoneConnected()) then
		PreGame.SetReady( Matchmaking.GetLocalID(), bChecked );
		Network.BroadcastPlayerInfo();
		UpdateDisplay();
	end
end
Controls.LocalReadyCheck:RegisterCheckHandler( OnReadyCheck );


-------------------------------------------------
-------------------------------------------------
function OnSelectTeam( selectionIndex, playerChoiceID )
	local playerID = GetPlayerIDBySelectionIndex(selectionIndex);
	PreGame.SetTeam(playerID, playerChoiceID);
	local slotInstance = m_SlotInstances[selectionIndex];
	local teamID = playerChoiceID + 1; -- Real programmers count from zero.
	if( slotInstance ~= nil ) then
		slotInstance.TeamLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_DEFAULT_TEAM_NAME", teamID );
	else
		Controls.TeamLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_DEFAULT_TEAM_NAME", teamID );
	end
	Network.BroadcastPlayerInfo();
	
	DoCheckTeams();
end


--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
function DoCheckTeams()

    m_bTeamsValid = false;
    local teamTest = PreGame.GetTeam( 0 );
    
    for i = 0, GameDefines.MAX_MAJOR_CIVS do
        if( PreGame.GetSlotStatus( i ) == SlotStatus.SS_COMPUTER or PreGame.GetSlotStatus( i ) == SlotStatus.SS_TAKEN ) then
        	if( PreGame.GetTeam( i ) ~= teamTest ) then
        	    m_bTeamsValid = true;
        	    break;
        	end
    	end
    end
    
end


-------------------------------------------------
-------------------------------------------------
function OnHandicapTeam( selectionIndex, handicap )
	local listingEntry = m_SlotInstances[selectionIndex];

	if(listingEntry ~= nil) then
		listingEntry.HandicapLabel:LocalizeAndSetText( GameInfo.HandicapInfos( "ID = '" .. handicap .. "'" )().Description );
	else
		Controls.HandicapLabel:LocalizeAndSetText( GameInfo.HandicapInfos( "ID = '" .. handicap .. "'" )().Description );
	end
	local playerID = GetPlayerIDBySelectionIndex(selectionIndex);
	PreGame.SetHandicap(playerID, handicap);
	Network.BroadcastPlayerInfo();
end


-------------------------------------------------
-- Refresh Player List
-------------------------------------------------
function RefreshPlayerList()
    m_HostID  = Matchmaking.GetHostID();
    m_bIsHost = Matchmaking.IsHost();
	m_bLaunchReady = true;
	
	-- Get the Current Player List
	local playerTable = Matchmaking.GetPlayerList();
	
	for i = 1, #m_SlotInstances do
		m_SlotInstances[i].Root:SetHide( true );
	end

	-- Display Each Player
	if( playerTable ) then
		for i, playerInfo in ipairs( playerTable ) do
		    local playerID = playerInfo.playerID;

			if( playerInfo.playerID == Matchmaking.GetLocalID() ) then
				UpdateLocalPlayer( playerInfo );
			else	
				UpdatePlayer( m_SlotInstances[ i ], playerInfo );
			end
			
			m_SlotInstances[i].playerID = playerTable[i].playerID;
		end
	end

	Controls.SlotStack:CalculateSize();
	Controls.SlotStack:ReprocessAnchoring();
	Controls.ListingScrollPanel:CalculateInternalSize();
end


-------------------------------------------------
-------------------------------------------------
function UpdatePlayer( slotInstance, playerInfo )
	-- Player Listing Entry
	if(slotInstance ~= nil) then
		slotInstance.Root:SetHide( false );

		local playerName = playerInfo.playerName or Locale.ConvertTextKey( "TXT_KEY_MULTIPLAYER_DEFAULT_PLAYER_NAME", playerID );
		local playerID   = playerInfo.playerID;

		--------------------------------------------------------------
		-- Handle Civ Info
		
        --slotInstance.Portrait
        
		local civIndex = PreGame.GetCivilization( playerID );
		if( civIndex ~= -1 ) then
			civ = GameInfo.Civilizations[ civIndex ];

			-- Use the Civilization_Leaders table to cross reference from this civ to the Leaders table
			local leader = GameInfo.Leaders( "Type = '" .. GameInfo.Civilization_Leaders( "CivilizationType = '" .. civ.Type .. "'" )().LeaderheadType .. "'" )();
			local leaderDescription = leader.Description;
			
			slotInstance.CivLabel:LocalizeAndSetText( "TXT_KEY_RANDOM_LEADER_CIV", Locale.ConvertTextKey( leaderDescription ), Locale.ConvertTextKey( civ.ShortDescription ) );
        
			IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, slotInstance.Portrait );
			SimpleCivIconHookup( playerID, 64, slotInstance.Icon);
		else   
			--------------------------------------------------------------
			-- Random Civ             
			slotInstance.CivLabel:LocalizeAndSetText( "TXT_KEY_RANDOM_LEADER" );

			IconHookup( 22, 64, "LEADER_ATLAS", slotInstance.Portrait );
			SimpleCivIconHookup(-1, 64, slotInstance.Icon);
		end

		--------------------------------------------------------------
		-- Set Team
		local teamID = PreGame.GetTeam(playerID) + 1; -- Real programmers count from zero.
		slotInstance.TeamLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_DEFAULT_TEAM_NAME", teamID );
		--------------------------------------------------------------
		
		--------------------------------------------------------------
		-- Set Handicap
		slotInstance.HandicapLabel:LocalizeAndSetText( GameInfo.HandicapInfos( "ID = '" .. PreGame.GetHandicap( playerID ) .. "'" )().Description );
		--------------------------------------------------------------

        local bIsHuman  = (PreGame.GetSlotStatus( playerID ) == SlotStatus.SS_TAKEN ) or
                          (PreGame.GetSlotStatus( playerID ) == SlotStatus.SS_OPEN );
        local bIsLocked = (PreGame.GetSlotClaim( playerID ) == SlotClaim.SLOTCLAIM_RESERVED ) or
                          (PreGame.GetSlotClaim( playerID ) == SlotClaim.SLOTCLAIM_ASSIGNED );

        local bIsDisabled = (PreGame.GetSlotStatus( playerID ) == SlotStatus.SS_CLOSED );
        local bIsReady;
        local bCantChange;

        if( bIsHuman ) then
            bIsReady = PreGame.IsReady( playerID );
            bCantChange = true; --(playerID == Matchmaking.GetLocalID()) and (not bIsReady);
            TruncateString(slotInstance.PlayerNameLabel, slotInstance.PlayerNameBox:GetSizeX() - 
						   slotInstance.PlayerNameLabel:GetOffsetX(), playerName); 
    		slotInstance.InvitePulldown:GetButton():SetText( playerName );
        else
            bIsReady =	(PreGame.IsReady( m_HostID )) or 
				(PreGame.GetLoadFileName() ~= "" and PreGame.GetSlotStatus( playerID ) == SlotStatus.SS_CLOSED); -- prevent closed player slots from blocking game startup when loading save games -tsmith
            
            if( m_bIsHost ) then
				if(PreGame:GetMultiplayerAIEnabled()) then
					bCantChange = bIsReady;
				else
					bCantChange = true;
				end
            else
                bCantChange = true;
            end
            
            TruncateString(slotInstance.PlayerNameLabel, slotInstance.PlayerNameBox:GetSizeX() - 
						   slotInstance.PlayerNameLabel:GetOffsetX(), Locale.ConvertTextKey("TXT_KEY_AI_NICKNAME")); 
    		slotInstance.InvitePulldown:GetButton():LocalizeAndSetText( "TXT_KEY_AI_NICKNAME" );
        end
        
        if( not bIsReady ) then
			m_bLaunchReady = false;
        end
        
        if( bIsDisabled ) then
        	slotInstance.EnableCheck:SetCheck( false );
        	slotInstance.EnableCheck:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_MP_ENABLE_SLOT" ) );	
    	else
        	slotInstance.EnableCheck:SetCheck( true );
        	slotInstance.EnableCheck:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_MP_DISABLE_SLOT" ) );
    	end

    	if( bIsLocked ) then
        	slotInstance.LockCheck:SetCheck( true );
        	slotInstance.LockCheck:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_MP_UNLOCK_SLOT" ) );
    	else
        	slotInstance.LockCheck:SetCheck( false );
            slotInstance.LockCheck:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_MP_LOCK_SLOT" ) );
        end

        slotInstance.ReadyHighlight:SetHide( not (bIsReady and not bIsDisabled) );

		slotInstance.PlayerNameLabel:SetHide( false );
	    slotInstance.InvitePulldown:SetHide( true );
		    
        slotInstance.CivPulldown:SetHide( bCantChange );
        slotInstance.TeamPulldown:SetHide( bCantChange );

		if ( bIsHuman ) then
			slotInstance.HandicapLabel:SetHide( false );
			slotInstance.HandicapPulldown:SetHide( bCantChange );
		else
			slotInstance.HandicapLabel:SetHide( true );
			slotInstance.HandicapPulldown:SetHide( true );
		end

        slotInstance.DisableBlock:SetHide( not bIsDisabled );
        
        if( m_bIsHost ) then
            slotInstance.EnableCheck:SetHide( bIsHuman or bCantChange );
            slotInstance.LockCheck:SetHide( bIsHuman or bCantChange );
            slotInstance.KickButton:SetHide( not bIsHuman );
        else
            slotInstance.EnableCheck:SetHide( true );
            slotInstance.LockCheck:SetHide( true );
            slotInstance.KickButton:SetHide( true );
        end
        
        --TEMP: S.S, hiding lock since it doesn't do anything and too many people believe it's the cause of MP probs.
        slotInstance.LockCheck:SetHide( true );
        
        slotInstance.HostIcon:SetHide( playerID ~= m_HostID );		
	end
end


-------------------------------------------------
-------------------------------------------------
function UpdateLocalPlayer( playerInfo )
	--------------------------------------------------------------
	-- Fill In Slot Info
	local playerName = playerInfo.playerName or Locale.ConvertTextKey( "TXT_KEY_MULTIPLAYER_DEFAULT_PLAYER_NAME", Matchmaking.GetLocalID() );
	
	Controls.RemoveButton:SetHide( true );
    --------------------------------------------------------------
    -- Get player civ
	local civIndex = PreGame.GetCivilization( Matchmaking.GetLocalID() );
	if( civIndex ~= -1 ) then
		--------------------------------------------------------------
		-- Selected a Civ
		civ = GameInfo.Civilizations[ civIndex ];

		-- Use the Civilization_Leaders table to cross reference from this civ to the Leaders table
		local leader = GameInfo.Leaders( "Type = '" .. GameInfo.Civilization_Leaders( "CivilizationType = '" .. civ.Type .. "'" )().LeaderheadType .. "'" )();
		local leaderDescription = leader.Description;
		
		Controls.CivLabel:LocalizeAndSetText( "TXT_KEY_RANDOM_LEADER_CIV", Locale.ConvertTextKey( leaderDescription ), Locale.ConvertTextKey( civ.ShortDescription ) );
			print(civIndex);
		IconHookup( leader.PortraitIndex, 64, leader.IconAtlas, Controls.Portrait );
		SimpleCivIconHookup( Matchmaking.GetLocalID(), 64, Controls.Icon);
	else   
		--------------------------------------------------------------
		-- Random Civ             
        
		Controls.CivLabel:LocalizeAndSetText( "TXT_KEY_RANDOM_LEADER" );
		IconHookup( 22, 64, "LEADER_ATLAS", Controls.Portrait );
		SimpleCivIconHookup(-1, 64, Controls.Icon);
	end
	--------------------------------------------------------------
	
	--------------------------------------------------------------
	-- Set Team
	local teamID = PreGame.GetTeam(Matchmaking.GetLocalID()) + 1; -- Real programmers count from zero.
	Controls.TeamLabel:LocalizeAndSetText( "TXT_KEY_MULTIPLAYER_DEFAULT_TEAM_NAME", teamID );
	--------------------------------------------------------------
	
	--------------------------------------------------------------
	-- Set Handicap
	Controls.HandicapLabel:LocalizeAndSetText( GameInfo.HandicapInfos( "ID = '" .. PreGame.GetHandicap( Matchmaking.GetLocalID() ) .. "'" )().Description );
	--------------------------------------------------------------
	
	-----------------------------------------------------------
	-- Ready Status
	local bIsReady = PreGame.IsReady( Matchmaking.GetLocalID() );
	if(PreGame.GetLoadFileName() ~= "") then
		Controls.LocalReadyCheck:SetHide(true);
		PreGame.SetReady(Matchmaking.GetLocalID(), true);
		if(not bIsReady) then
			Network.BroadcastPlayerInfo();
		end
		bIsReady = true;
	else
		Controls.LocalReadyCheck:SetHide(false);
	end
	if( not bIsReady ) then
		m_bLaunchReady = false;
	end

    Controls.LocalReadyCheck:SetCheck( bIsReady );
    Controls.ReadyHighlight:SetHide( not bIsReady );

	TruncateString(Controls.PlayerNameLabel, Controls.PlayerNameBox:GetSizeX() - 
				   Controls.PlayerNameLabel:GetOffsetX(), playerName); 
	Controls.CivPulldown:SetHide( bIsReady );
	Controls.TeamPulldown:SetHide( bIsReady );
	Controls.HandicapPulldown:SetHide( bIsReady );
    Controls.HostIcon:SetHide( Matchmaking.GetLocalID() ~= m_HostID );
	---------------------------------------------------------
	
end



-------------------------------------------------
-- Back Button Handler
-------------------------------------------------
function BackButtonClick()
	HandleExitRequest();
end
Controls.BackButton:RegisterCallback( Mouse.eLClick, BackButtonClick );


-------------------------------------------------
-- Leave the Game
-------------------------------------------------
function HandleExitRequest()

	local isHost = Matchmaking.IsHost();
	Matchmaking.LeaveMultiplayerGame();
	-- Refresh the lobby as we enter it
	if ( PreGame.IsInternetGame() and not isHost ) then
		Matchmaking.RefreshInternetGameList();
	elseif ( not PreGame.IsInternetGame() and not isHost ) then
		Matchmaking.RefreshLANGameList();
	end

	UIManager:DequeuePopup( ContextPtr );
end


----------------------------------------------------------------
-- Connection Established
----------------------------------------------------------------
function OnConnect( playerID )
    if( ContextPtr:IsHidden() == false ) then

    	if Matchmaking.IsHost() then
    		Network.BroadcastGameSettings();
    	end
    	
    	UpdateDisplay();
    	Network.BroadcastPlayerInfo();

        BuildPlayerNames();
    	OnChat( playerID, Locale.ConvertTextKey( "TXT_KEY_CONNECTED" ) );

    end
end
Events.ConnectedToNetworkHost.Add( OnConnect );


----------------------------------------------------------------
----------------------------------------------------------------
function OnDisconnectOrPossiblyUpdate()
	UpdateDisplay();
end
Events.MultiplayerGamePlayerUpdated.Add( OnDisconnectOrPossiblyUpdate );


----------------------------------------------------------------
----------------------------------------------------------------
function OnDisconnect( playerID )
    if( ContextPtr:IsHidden() == false ) then
    	OnChat( playerID, Locale.ConvertTextKey( "TXT_KEY_DISCONNECTED" ) );
	end
end
Events.MultiplayerGamePlayerDisconnected.Add( OnDisconnect );


----------------------------------------------------------------
-- UPDATE CURRENT SETTINGS
----------------------------------------------------------------
function UpdateDisplay()
    if( ContextPtr:IsHidden() == false ) then

	UpdateOptions();
    RefreshPlayerList();
    DoCheckTeams();
	if(m_bIsHost) then
		Controls.LaunchButton:SetHide( false );
		if( not Network.IsEveryoneConnected() ) then
			m_bLaunchReady = false;
		end

		Controls.LaunchButton:SetDisabled( not (m_bLaunchReady and m_bTeamsValid) );
	
	    if( m_bTeamsValid ) then
            Controls.LaunchButton:SetToolTipString();
		else
            Controls.LaunchButton:LocalizeAndSetToolTip( "TXT_KEY_BAD_TEAMS" );
        end
        
	else
		Controls.LaunchButton:SetHide( true );
	end
    end
end


-------------------------------------------------
-------------------------------------------------
function OnPreGameDirty()
    UpdateDisplay();
end
Events.PreGameDirty.Add( OnPreGameDirty );


----------------------------------------------------------------        
-- Input Handler
----------------------------------------------------------------        
function InputHandler( uiMsg, wParam, lParam )
	if uiMsg == KeyEvents.KeyDown then
		if wParam == Keys.VK_ESCAPE then
			HandleExitRequest();
			return true;
		end
	end
	
    if( uiMsg == KeyEvents.KeyUp
        and (wParam == Keys.VK_TAB or wParam == Keys.VK_RETURN)) then
        Controls.ChatEntry:TakeFocus();
        return true;
    end
end
ContextPtr:SetInputHandler( InputHandler );


-------------------------------------------------
-------------------------------------------------
function ShowHideHandler( bIsHide, bIsInit )

    if( bIsHide ) then
        Controls.ChatStack:DestroyAllChildren();
    end
    
    if( not bIsHide ) then
		
        BuildPlayerNames();
		UpdateDisplay();
	end

end
ContextPtr:SetShowHideHandler( ShowHideHandler );


-------------------------------------------------
-------------------------------------------------
function UpdateOptions()

	-- Set Game Name
	local strGameName = Matchmaking.GetCurrentGameName();
	Controls.NameLabel:SetText( strGameName );
	
	if( PreGame.GetLoadFileName() ~= "" ) then
	    Controls.LoadingBox:SetHide( false );
    else
	    Controls.LoadingBox:SetHide( true );
	end

	-- Set Max Turns if set
	local maxTurns = PreGame.GetMaxTurns();
	if(maxTurns == 0) then
		Controls.MaxTurns:SetHide(true);
	else
		Controls.MaxTurns:SetHide(false);
		Controls.MaxTurns:SetText(Locale.ConvertTextKey("TXT_KEY_MAX_TURNS", maxTurns));
	end

	-- Set Start Era
--	Controls.StartEraLabel:SetText( Locale.ConvertTextKey("TXT_KEY_START_ERA", Locale.ConvertTextKey(GameInfo.Eras[ PreGame.GetEra() ].Description)));
	Controls.StartEraLabel:SetText( Locale.ConvertTextKey( GameInfo.Eras[ PreGame.GetEra() ].Description ) );

	-- Set Game Map Type
	if( not PreGame.IsRandomMapScript() ) then   
		local mapScriptFileName = PreGame.GetMapScript();
		local mapScript = nil;
        
		for row in GameInfo.MapScripts() do
			if(row.FileName == mapScriptFileName) then
				mapScript = row;
				break;
			end
		end
		
		if(mapScript ~= nil) then
			Controls.MapTypeLabel:LocalizeAndSetText( mapScript.Name );
			Controls.MapTypeLabel:LocalizeAndSetToolTip( mapScript.Description );
		else
			PreGame.SetRandomMapScript(true);
		end
	end
    
	if(PreGame.IsRandomMapScript()) then
		Controls.MapTypeLabel:LocalizeAndSetText( "TXT_KEY_RANDOM_MAP_SCRIPT" );
		Controls.MapTypeLabel:LocalizeAndSetToolTip( "TXT_KEY_RANDOM_MAP_SCRIPT_HELP" );
	end
	
	-- Set Map Size
	if(not PreGame.IsRandomWorldSize() ) then
		info = GameInfo.Worlds[ PreGame.GetWorldSize() ];
		Controls.MapSizeLabel:LocalizeAndSetText( info.Description );
		Controls.MapSizeLabel:LocalizeAndSetToolTip( info.Help );
	else
		Controls.MapSizeLabel:LocalizeAndSetText( "TXT_KEY_RANDOM_MAP_SIZE" );
		Controls.MapSizeLabel:LocalizeAndSetToolTip( "TXT_KEY_RANDOM_MAP_SIZE_HELP" );
	end
	
    -- Set Game Pace Slot
    info = GameInfo.GameSpeeds[ PreGame.GetGameSpeed() ];
	Controls.GameSpeedLabel:LocalizeAndSetText( info.Description );
	Controls.GameSpeedLabel:LocalizeAndSetToolTip( info.Help );
	
	-- Game Options
	g_AdvancedOptionIM:ResetInstances();
	g_AdvancedOptionsList = {};
	
    local count = 1;
    -- When there's 8 or more players connected to us (9 player game), warn that it's an unsupported game.
    
    local totalPlayers = 0;
    for i = 0, GameDefines.MAX_MAJOR_CIVS do
        if( PreGame.GetSlotStatus( i ) == SlotStatus.SS_COMPUTER or PreGame.GetSlotStatus( i ) == SlotStatus.SS_TAKEN ) then
			if( PreGame.GetSlotClaim( i ) == SlotClaim.SLOTCLAIM_ASSIGNED ) then
				totalPlayers = totalPlayers + 1;
			end
		end
	end
	
    if(totalPlayers > 8) then
		local controlTable = g_AdvancedOptionIM:GetInstance();
		g_AdvancedOptionsList[count] = controlTable;
		controlTable.Text:LocalizeAndSetText("TXT_KEY_MULTIPLAYER_UNSUPPORTED_NUMBER_OF_PLAYERS");
		count = count + 1;
    end
    
    for option in GameInfo.GameOptions{Visible = 1} do		
		local savedValue = PreGame.GetGameOption(option.Type);
		if(savedValue ~= nil and savedValue == 1) then
			local controlTable = g_AdvancedOptionIM:GetInstance();
			g_AdvancedOptionsList[count] = controlTable;
			controlTable.Text:LocalizeAndSetText(option.Description);
			count = count + 1;
		end
	end
end


-------------------------------------------------
-------------------------------------------------
function PopulateCivPulldown( pullDown, playerID )

    local controlTable = {};

    ------------------------------------------------------------------------------------------------
    -- set up the random slot
    ------------------------------------------------------------------------------------------------
    pullDown:BuildEntry( "InstanceOne", controlTable );
    controlTable.Button:SetVoids( playerID, -1 );

    controlTable.Button:SetText( Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER" ) );
    controlTable.Button:SetToolTipString( Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER_HELP" ) );


    for civ in GameInfo.Civilizations( "Playable = 1" ) do

        -- Use the Civilization_Leaders table to cross reference from this civ to the Leaders table
        leaderDescription = GameInfo.Leaders( "Type = '" .. GameInfo.Civilization_Leaders( "CivilizationType = '" .. civ.Type .. "'" )().LeaderheadType .. "'" )().Description;
        
        controlTable = {};
        pullDown:BuildEntry( "InstanceOne", controlTable );
        
        controlTable.Button:SetVoids( playerID, civ.ID );
        controlTable.Button:SetText( Locale.ConvertTextKey( "TXT_KEY_RANDOM_LEADER_CIV", Locale.ConvertTextKey( leaderDescription ), Locale.ConvertTextKey( civ.ShortDescription ) ) );
        controlTable.Button:SetToolTipString( Locale.ConvertTextKey( civ.Description ) );

    end    
    pullDown:CalculateInternals();
    pullDown:RegisterSelectionCallback( CivSelected );
end


-------------------------------------------------
-- TODO: This only gets called once.  We need something more dynamic.
-------------------------------------------------
function PopulateInvitePulldown( pullDown, playerID )

	local controlTable = {};
	pullDown:BuildEntry( "InstanceOne", controlTable );

	controlTable.Button:SetText( Locale.ConvertTextKey("TXT_KEY_AI_NICKNAME") );
	controlTable.Button:SetVoids( playerID, -1 );

	local friendList = Matchmaking.GetFriendList();

	if friendList then

		-- Populate with Steam friends.
		for i = 1, #friendList do

			local controlTable = {};
			pullDown:BuildEntry( "InstanceOne", controlTable );

			controlTable.Button:SetText( "Invite " .. friendList[i].playerName );
			controlTable.Button:SetVoids( playerID, friendList[i].steamID );

		end

	end

	pullDown:CalculateInternals();
	pullDown:RegisterSelectionCallback( InviteSelected );

end


-------------------------------------------------
-------------------------------------------------
function PopulateTeamPulldown( pullDown, playerID )

	local playerTable = Matchmaking.GetPlayerList();
	local count = 0;

	pullDown:ClearEntries();

	-- Display Each Player
	if playerTable then
    	count = #playerTable;
		
		for i = 1, count, 1 do

			local controlTable = {};
			pullDown:BuildEntry( "InstanceOne", controlTable );
			
			controlTable.Button:SetText( Locale.ConvertTextKey("TXT_KEY_MULTIPLAYER_DEFAULT_TEAM_NAME", i) );
			controlTable.Button:SetVoids( playerID, i-1 ); -- TODO: playerID is really more like the slot position.

		end

		pullDown:CalculateInternals();
		pullDown:RegisterSelectionCallback( OnSelectTeam );
	end

end


-------------------------------------------------
-------------------------------------------------
function PopulateHandicapPulldown( pullDown, playerID )
	for info in GameInfo.HandicapInfos() do
		local controlTable = {};
		pullDown:BuildEntry( "InstanceOne", controlTable );

		controlTable.Button:SetText( Locale.ConvertTextKey(info.Description) );
		controlTable.Button:SetVoids( playerID, info.ID );
	end    

	pullDown:CalculateInternals();
	pullDown:RegisterSelectionCallback( OnHandicapTeam );

end



--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------
-- Chat
--------------------------------------------------------------------------------------------------------------------------------
--------------------------------------------------------------------------------------------------------------------------------

-------------------------------------------------
-------------------------------------------------
local bFlipper = false;
function OnChat( fromPlayer, text )

    local controlTable = {};
    ContextPtr:BuildInstanceForControl( "ChatEntry", controlTable, Controls.ChatStack );
    
    table.insert( g_ChatInstances, controlTable );
    if( #g_ChatInstances > 100 ) then
    
        Controls.ChatStack:ReleaseChild( g_ChatInstances[ 1 ].Box );
        table.remove( g_ChatInstances, 1 );
    end
    
    controlTable.String:SetText( m_PlayerNames[ fromPlayer ] .. " : " .. text );
    controlTable.Box:SetSizeY( controlTable.String:GetSizeY() + 15 );
    controlTable.Box:ReprocessAnchoring();

    if( bFlipper ) then
        controlTable.Box:SetColorChannel( 3, 0.4 );
    end
    bFlipper = not bFlipper;

    Controls.ChatStack:CalculateSize();
    Controls.ChatScroll:CalculateInternalSize();
    Controls.ChatScroll:SetScrollValue( 1 );
end
Events.GameMessageChat.Add( OnChat );


-------------------------------------------------
-------------------------------------------------
function SendChat( text )
    if( string.len( text ) > 0 ) then
        Network.SendChat( text );
    end
    Controls.ChatEntry:ClearString();
end
Controls.ChatEntry:RegisterCallback( SendChat );


----------------------------------------------------------------
----------------------------------------------------------------
function OnEnable( bIsCheck, slotNumber )
	local playerID = GetPlayerIDBySelectionIndex( slotNumber );

	local bCanEnableAISlots = PreGame.GetMultiplayerAIEnabled();
	
	if(bCanEnableAISlots) then
		if( bIsCheck ) then
    		PreGame.SetSlotStatus( playerID, SlotStatus.SS_COMPUTER );
		else
    		PreGame.SetSlotStatus( playerID, SlotStatus.SS_CLOSED );
		end
	end
    UpdateDisplay();
    Network.BroadcastPlayerInfo();
end


----------------------------------------------------------------
----------------------------------------------------------------
function OnLock( bIsCheck, slotNumber )

	local playerID = GetPlayerIDBySelectionIndex( slotNumber );
	
    if( bIsCheck ) then
    	PreGame.SetSlotClaim( playerID, SlotClaim.SLOTCLAIM_RESERVED );
	else
    	PreGame.SetSlotClaim( playerID, SlotClaim.SLOTCLAIM_UNASSIGNED );
	end

    UpdateDisplay();
end


----------------------------------------------------------------
-- build the slots
----------------------------------------------------------------
for i = 1, GameDefines.MAX_MAJOR_CIVS, 1 do

    local instance = {};
    ContextPtr:BuildInstanceForControl( "PlayerSlot", instance, Controls.SlotStack );
    instance.Root:SetHide( true );
    
    PopulateCivPulldown( instance.CivPulldown, i );
    PopulateInvitePulldown( instance.InvitePulldown, i );
    PopulateTeamPulldown( instance.TeamPulldown, i );
    PopulateHandicapPulldown( instance.HandicapPulldown, i );

    instance.LockCheck:RegisterCheckHandler( OnLock );
    instance.EnableCheck:RegisterCheckHandler( OnEnable );
    instance.EnableCheck:SetVoid1( i );
    instance.KickButton:RegisterCallback( Mouse.eLClick, OnKickPlayer );
    instance.KickButton:SetVoid1( i );
    
    m_SlotInstances[i] = instance;
end

-- Setup Local Slot
PopulateCivPulldown( Controls.CivPulldown, 0 );
PopulateTeamPulldown( Controls.TeamPulldown, 0 );
PopulateHandicapPulldown( Controls.HandicapPulldown, 0 );


-----------------------------------------------------------------
-- Adjust for resolution
-----------------------------------------------------------------
function AdjustScreenSize()
    local _, screenY = UIManager:GetScreenSizeVal();
    
    local TOP_COMPENSATION = 52 + ((screenY - 768) * 0.3 );
    local TOP_FRAME = 108;
    local BOTTOM_COMPENSATION = 240;
    local LOCAL_SLOT_COMPENSATION = 80;

    Controls.MainGrid:SetSizeY( screenY - TOP_COMPENSATION );
    
    Controls.ListingScrollPanel:SetSizeY( screenY - TOP_COMPENSATION - LOCAL_SLOT_COMPENSATION - BOTTOM_COMPENSATION - TOP_FRAME );
    Controls.ListingScrollPanel:CalculateInternalSize();
    
    Controls.VerticalTrim:SetSizeY( screenY - TOP_COMPENSATION - BOTTOM_COMPENSATION - TOP_FRAME );
end


-------------------------------------------------
-------------------------------------------------
function OnUpdateUI( type )
    if( type == SystemUpdateUIType.ScreenResize ) then
        AdjustScreenSize();
    end
end
Events.SystemUpdateUI.Add( OnUpdateUI );

function OnAbandoned()
	HandleExitRequest();
end
Events.MultiplayerGameAbandoned.Add( OnAbandoned );
AdjustScreenSize();
